@@ -13,11 +13,11 @@ import yaml
 from apispec import APISpec, yaml_utils
 from apispec.ext.marshmallow import MarshmallowPlugin
 from asgiref.wsgi import WsgiToAsgi
-from starlette.exceptions import ExceptionMiddleware
-from starlette.lifespan import LifespanHandler
+from starlette.middleware.errors import ServerErrorMiddleware
 from starlette.middleware.cors import CORSMiddleware
 from starlette.middleware.gzip import GZipMiddleware
 from starlette.middleware.httpsredirect import HTTPSRedirectMiddleware
+from starlette.middleware.lifespan import LifespanMiddleware
 from starlette.middleware.trustedhost import TrustedHostMiddleware
 from starlette.routing import Router
 from starlette.staticfiles import StaticFiles
@@ -90,6 +90,7 @@ class API:
         self.hsts_enabled = enable_hsts
         self.cors = cors
         self.cors_params = cors_params
+        self.debug = debug
 
         if not allowed_hosts:
             # if not debug:
@@ -135,11 +136,11 @@ class API:
 
         self.add_middleware(TrustedHostMiddleware, allowed_hosts=self.allowed_hosts)
 
-        self.lifespan_handler = LifespanHandler()
+        self.lifespan_handler = LifespanMiddleware(self.app)
 
         if self.cors:
             self.add_middleware(CORSMiddleware, **self.cors_params)
-        self.add_middleware(ExceptionMiddleware, debug=debug)
+        self.add_middleware(ServerErrorMiddleware, debug=debug)
 
         # Jinja enviroment
         self.jinja_env = jinja2.Environment(
@@ -196,6 +197,7 @@ class API:
         self.app = middleware_cls(self.app, **middleware_config)
 
     def __call__(self, scope):
+
         if scope["type"] == "lifespan":
             return self.lifespan_handler(scope)
 
@@ -220,14 +222,41 @@ class API:
         async def asgi(receive, send):
             nonlocal scope, self
 
-            req = models.Request(scope, receive=receive, api=self)
-            resp = await self._dispatch_request(
-                req, scope=scope, send=send, receive=receive
-            )
-            await resp(receive, send)
+            if scope["type"] == "lifespan":
+                return self.lifespan_handler(scope)
+            elif scope["type"] == "websocket":
+                ws = WebSocket(scope=scope, receive=receive, send=send)
+                await self._dispatch_ws(ws)
+            else:
+                req = models.Request(scope, receive=receive, api=self)
+                resp = await self._dispatch_request(
+                    req, scope=scope, send=send, receive=receive
+                )
+                await resp(receive, send)
 
         return asgi
 
+    async def _dispatch_ws(self, ws):
+        route = self.path_matches_route(ws.url.path)
+        route = self.routes.get(route)
+        # await self._dispatch(route, ws=ws)
+        try:
+            try:
+                # Run the view.
+                r = self.background(route.endpoint, ws)
+                # If it's async, await it.
+                if hasattr(r, "cr_running"):
+                    await r
+            except TypeError as e:
+                cont = True
+        except Exception:
+            self.background(
+                self.default_response,
+                websocket=route.uses_websocket,
+                error=True
+            )
+            raise
+
     def add_schema(self, name, schema, check_existing=True):
         """Adds a mashmallow schema to the API specification."""
         if check_existing:
@@ -292,10 +321,7 @@ class API:
         route = self.path_matches_route(req.url.path)
         route = self.routes.get(route)
         if route:
-            if route.uses_websocket:
-                resp = WebSocket(**options)
-            else:
-                resp = models.Response(req=req, formats=self.formats)
+            resp = models.Response(req=req, formats=self.formats)
 
             for before_request in self.before_requests:
                 await self._execute_route(route=before_request, req=req, resp=resp)
@@ -303,8 +329,8 @@ class API:
             await self._execute_route(route=route, req=req, resp=resp, **options)
         else:
             resp = models.Response(req=req, formats=self.formats)
-            self.default_response(req, resp, notfound=True)
-        self.default_response(req, resp)
+            self.default_response(req=req, resp=resp, notfound=True)
+        self.default_response(req=req, resp=resp)
 
         self._prepare_session(resp)
         self._prepare_cookies(resp)
@@ -315,11 +341,12 @@ class API:
 
         params = route.incoming_matches(req.url.path)
 
+        cont = True
+
         if route.is_function:
             try:
                 try:
                     # Run the view.
-
                     r = self.background(route.endpoint, req, resp, **params)
                     # If it's async, await it.
                     if hasattr(r, "cr_running"):
@@ -414,12 +441,17 @@ class API:
             sorted(self.routes.items(), key=lambda item: item[1]._weight())
         )
 
-    def default_response(self, req, resp, notfound=False, error=False):
+    def default_response(
+        self, req=None, resp=None, websocket=False, notfound=False, error=False
+    ):
+        if websocket:
+            return
+
         if resp.status_code is None:
             resp.status_code = 200
 
         if self.default_endpoint and notfound:
-            self.default_endpoint(req, resp)
+            self.default_endpoint(req=req, resp=resp)
         else:
             if notfound:
                 resp.status_code = status_codes.HTTP_404
@@ -433,7 +465,7 @@ class API:
 
     def static_response(self, req, resp):
         index = (self.static_dir / "index.html").resolve()
-        resp.content = ""
+        resp.content = None
         if os.path.exists(index):
             with open(index, "r") as f:
                 resp.text = f.read()
@@ -533,7 +565,7 @@ class API:
         # TODO: Absolute_url
         """Given an endpoint, returns a rendered URL for its route.
 
-        :param view: The route endpoint you're searching for.
+        :param endpoint: The route endpoint you're searching for.
         :param params: Data to pass into the URL generator (for parameterized URLs).
         """
         route_object = self._route_for(endpoint)
@@ -628,4 +660,6 @@ class API:
         spawn()
 
     def run(self, **kwargs):
+        if 'debug' not in kwargs:
+            kwargs.update({'debug': self.debug})
         self.serve(**kwargs)
